package ANEP;

/*
 * $Id: ANEPd.java,v 1.5 1997/11/21 21:38:01 mwh Exp $
 * $Revision: 1.5 $
 *
 * Copyright 1997 by Carl A. Gunter, Mike Hicks, Pankaj Kakkar,
 * Jonathan Moore, Scott Nettles
 *
 */

import Log.Log;
import java.net.*;
import java.io.*;

class PLANHandler implements Handler {
  static int PLANportNo = 3324;
  
  public void handle (byte [] payload, TLVList TLVs) {
    
    try {
      byte [] data =
	(new ANEPacket (ANEPacket.PLANtypeId, TLVs, payload)).marshall ();
      DatagramPacket p =
	new DatagramPacket (data,
			    data.length,
			    InetAddress.getLocalHost (),
			    PLANportNo);
      
      ANode.sock.send (p);
    } catch (IOException e) {
      // Throw the packet
    }
  }
}

class ANTSHandler implements Handler {
  static int ANTSportNo = 3323;
  
  public void handle (byte [] payload, TLVList TLVs) {
    
    try {
      byte [] data =
	(new ANEPacket (ANEPacket.ANTStypeId, TLVs, payload)).marshall ();
      DatagramPacket p =
	new DatagramPacket (data,
			    data.length,
			    InetAddress.getLocalHost (),
			    ANTSportNo);
      
      ANode.sock.send (p);
    } catch (IOException e) {
      // Throw the packet
    }
  }
}

class ANEPd {
  static int ANEPportNo = 3322;
  
  static PLANHandler ph = new PLANHandler ();
  static ANTSHandler ah = new ANTSHandler ();
  
  static void usage() {
    System.out.println("Usage : java Main [-v|-d|-q] [-p port]");
  }
  
  static void parseArgs (String [] args) {
    int i;
    
    for (i=0; i<args.length; i++) {
      if (args[i].equals ("-?")) {
	usage();
	System.exit(1);
      }
      
      else if (args[i].equals ("-v")) {
	Log.setLevel(Log.VERBOSE_LEVEL);
	Log.println("Main: verbose mode on.");
      }
      
      else if (args[i].equals ("-d")) {
	Log.setLevel(Log.DEBUG_LEVEL);
	Log.println("Main: debug mode on.");
      }
      
      else if (args[i].equals ("-q"))
	Log.setLevel(Log.QUIET_LEVEL);
      
      else if (args[i].equals ("-p")) {
	i++;
	ANEPportNo = Integer.valueOf (args[i]).intValue ();
      }
      
      else {
	System.out.println ("Invalid argument : " + args[i]);
	usage ();
      }
    }
  }
  
  public static void main (String [] args) throws SocketException {
    parseArgs (args);
    
    ANode.start (ANEPportNo,
		 DispatchPairList.Cons 
		 (new DispatchPair (ANEPacket.PLANtypeId, ph),
		  DispatchPairList.Cons 
		  (new DispatchPair (ANEPacket.ANTStypeId, ah),
		   DispatchPairList.Nil)));

    ANode.run ();
  }
}
/*
 * $Log: ANEPd.java,v $
 * Revision 1.5  1997/11/21 21:38:01  mwh
 * Fixed marshalling to respect network byte order.  Added additional
 * classes to support ANON.
 *
 *
 */
