package ANEP;

/*
 * $Id: ANode.java,v 1.10 1997/11/21 22:30:35 mwh Exp $
 * $Revision: 1.10 $
 *
 * Copyright 1997 by Carl A. Gunter, Mike Hicks, Pankaj Kakkar,
 * Jonathan Moore, Scott Nettles
 *
 */

import Log.Log;

import java.net.DatagramSocket;
import java.net.DatagramPacket;
import java.net.InetAddress;
import java.net.SocketException;
import java.net.UnknownHostException;
import java.io.IOException;

final public class ANode {
  static public DatagramSocket sock;
  static int port;
  static DispatchPairList handlerList;
  
  static DatagramPacket nopPkt = null;
  
    public static void start (int p,
			      DispatchPairList h) throws SocketException {
      port = p;
      sock = new DatagramSocket (port);
      handlerList = h;
    }

  public static void defaultHandler (ANEPacket anPack) {
    TLV destTLV;
    
    if (anPack.getFlags () >> 7 == 0) {
      // Check MSB
      try {
	destTLV = anPack.getTLVs ().find (2);
	} catch (TLVNotFoundException e) {
	  destTLV = null;
	}
	
	if (destTLV != null) {
	  // InetAddress?? What about other schemes?
	}
    }
  }
  
  public static void run () {
    DispatchPairList temp;
    byte[] inbuf = new byte [65535];
    
    while (true) {
      try {
	DatagramPacket p = new DatagramPacket (inbuf, 65535);
	sock.receive (p);
	if (p.getLength() == 0) { // ICMP error hack
	  continue;
	}
	
	ANEPacket anPack = ANEPacket.unmarshall (p.getData ());
	
	Log.println ("ANode: IN: " + anPack);
	
	int typeId = anPack.getTypeId ();
	try {
	  Handler handler =
	    handlerList.find (typeId).handler;
	  
	  Log.println
	    ("ANode: Found handler for type ID = " + typeId);
	  
	  handler.handle (anPack.getPayload (), anPack.getTLVs ());
	} catch (PairNotFoundException e) {
	  Log.println
	    ("ANode: Not found any handler : invoking default handler");
	  
	  defaultHandler (anPack); break;
	}
      } catch (BadANEPversionException e) {
	Log.println("ANode: Received unsupported ANEP version exception:"+e);	
      } catch (IOException e) {
	// The folln mouse copied from ActiveRouter code.
	
	if ((e.getMessage ()).equals ("Interrupted system call")) {
	  // just retry on EINTR
	} else {
	  // got some kind of error
	  System.out.print("DatagramListenThread: ");
	  System.out.print(e);
	  if ((e.getMessage()).equals("Connection refused")) {
	    // Workaround; I think this may be an error in the JDK1.1
	    // implementation of DatagramSockets, since they should
	    // be connectionless. Will look into this further.
	    System.out.println(": Attempting to ignore.");
	  } else {		  
	    System.out.println(": Fatal error.");
	    System.exit(1);
	  }
	}
      }
    }
  } // run

  public static void sendPkt (ANEPacket p, InetAddress dest) 
       throws IOException {
	 
    Log.println ("ANode: OUT to " + dest);
    Log.println (Log.DEBUG_LEVEL,": "+p.toString());

    byte [] data = p.marshall ();
    DatagramPacket toSend = 
      new DatagramPacket (data, data.length, dest, port);
    
    try {
      if (nopPkt == null) {
	// the following would allow the creation of a PLAN packet which
	// essentially does nothing on execution. However, the empty
	// packet which is actually used further down seems to work ok
	// for clearing the pending exceptions
	
	byte [] nopPktData = ("").getBytes();
	nopPkt = 
	  new DatagramPacket(nopPktData,
			     nopPktData.length,
			     InetAddress.getLocalHost(),
			     port);
      }
      // clear out any pending exceptions with the nopPkt
      sock.send(nopPkt);
    } catch (UnknownHostException uhe) {
      System.out.println("ANode.sendPkt: " + uhe);
      System.exit(1);
    } catch (IOException s) {
      // don't report if this guy dies
    }
    try {
      // then send the real packet
      sock.send(toSend);
    } catch (IOException ioe) {
      System.out.println("ANode.sendPkt: " + ioe +
			 "\n  (dropping packet)");
    }
    Log.println ("ANode: OUT: succeeded");
  }
}


/*
 * $Log: ANode.java,v $
 * Revision 1.10  1997/11/21 22:30:35  mwh
 * removed extraneous exception
 *
 * Revision 1.9  1997/11/21 21:38:01  mwh
 * Fixed marshalling to respect network byte order.  Added additional
 * classes to support ANON.
 *
 *
 */
