package ANEP;

import java.io.*;
import java.net.InetAddress;

/*
 * This file contains routines to serialize basic Java types
 * by writing them to an output stream.  All items are written
 * in network-byte order (that is, MSB first).
 *
 * Copyright 1997 by Carl A. Gunter, Mike Hicks, Pankaj Kakkar,
 * Jonathan Moore, Scott Nettles, and Sam Weber
 *
 * $Id: Marshalling.java,v 1.6 1997/11/21 21:38:01 mwh Exp $
 * $Revision: 1.6 $
 */

public class Marshalling {

  /*
   *  Integers
   */

  public static void putInt (OutputStream os, int i) throws IOException {
    int int0 = i & 0xff; i = i>>8;
    int int1 = i & 0xff; i = i>>8;
    int int2 = i & 0xff; i = i>>8;
    int int3 = i & 0xff;
    
    os.write (int3); os.write (int2); os.write (int1); os.write (int0);
  }

  public static int getInt (InputStream is) throws IOException {
    int int3 = is.read ();
    int int2 = is.read ();
    int int1 = is.read ();
    int int0 = is.read ();
    
    int i = int3;
    i *= 256; i += int2;
    i *= 256; i += int1;
    i *= 256; i += int0;
    
    return i;
  }

  public static void putAsShort (OutputStream os, int i) throws IOException {
    int int0 = (i & 0xff); i = i>>8;
    int int1 = (i & 0xff);
    
    os.write (int1); os.write (int0);
  }

  public static int getAsShort (InputStream is) throws IOException {
    int int1 = is.read ();
    int int0 = is.read ();
    
    int i = (short)int1;
    i *= 256; i += int0;
    
    return i;
  }  

  public static void putAsByte (OutputStream os, int i) throws IOException {
    int int0 = (i & 0xff);
    os.write (int0);
  }

  public static int getAsByte (InputStream is) throws IOException {
    int int0 = is.read ();
    return int0;
  }

  /* 
   *  Short
   */

  public static void putShort (OutputStream os, short i) throws IOException {
    int x = i;
    int int0 = (x & 0xff); x = x>>8;
    int int1 = (x & 0xff);
    
    os.write (int1); os.write (int0);
  }
  
  public static short getShort (InputStream is) throws IOException {
    int int1 = is.read ();
    int int0 = is.read ();
    
    short i = (short)int1;
    i *= 256; i += int0;
    
    return i;
  }

  /*
   *  String
   */
  
  public static void putString (OutputStream os, String s) throws IOException {
    int i;
    
    putInt (os, s.length ());
    for (i=0; i<s.length (); i++)
      putChar (os, s.charAt (i));
  }
  
  public static String getString (InputStream is) throws IOException {
    int length = getInt (is);
    int i; String retValue = "";
    
    for (i=0; i<length; i++)
      retValue += getChar (is);
    
    return retValue;
  }
  
  public static void putChar (OutputStream os, char c) throws IOException {
    os.write ((byte)c);
  }
  
  public static char getChar (InputStream is) throws IOException {
    return (char)(is.read ());
  }

  /*
   *  Boolean
   */

  public static void putBoolean (OutputStream os, boolean b) throws IOException {
    if (b)
      os.write (1);
    else
      os.write (0);
  }
  
  public static boolean getBoolean (InputStream is) throws IOException {
    int i = is.read ();
    
    if (i==0)
      return false;
    else
      return true;
  }

  /*
   *  Byte
   */
  
  public static void putByte (OutputStream os, byte b) throws IOException {
    os.write (b);
  }
  
  public static byte getByte (InputStream is) throws IOException {
    return (byte)(is.read ());
  }


  /*
   *  InetAddress
   */

  public static void putInetAddress (OutputStream os, InetAddress addr)
       throws IOException {
	 
    // write the address
    byte [] ipAddr = addr.getAddress ();
    os.write(ipAddr[0]);
    os.write(ipAddr[1]);
    os.write(ipAddr[2]);
    os.write(ipAddr[3]);
  }

  public static InetAddress getInetAddress(InputStream is) 
       throws IOException {

    String s = "";
    for (int i=0; i<3; i++)
      s += (getAsByte(is) + ".");
    s += getAsByte(is);

    try {
      return InetAddress.getByName(s);
    } catch (java.net.UnknownHostException e) {
      throw new IOException("Failed to unmarshall inetaddress: "+e);
    }
  }

  // Test code to make sure what we put in is the same as what we
  // get out
  public static void main(String args[]) throws IOException {
    int i = 0xffeeddcc;
    short j = (short)0xddcc;
    int ji = 0xddcc;
    byte k = (byte)0xff;
    int ki = 0xff;
    
    ByteArrayOutputStream os = new ByteArrayOutputStream ();
    
    putInt(os,i);
    putShort(os,j);
    putAsShort(os,ji);
    putAsByte(os,ki);
    putString(os,"hello");
    putBoolean(os,true);
    putChar(os,'a');

    ByteArrayInputStream is = new ByteArrayInputStream(os.toByteArray());

    int i1 = getInt(is);
    short j1 = getShort(is);
    int j2 = getAsShort(is);
    int k1 = getAsByte(is);
    String s = getString(is);
    boolean b = getBoolean(is);
    char a = getChar(is);

    System.out.println(i+" =? "+i1);
    System.out.println(j+" =? "+j1);
    System.out.println(ji+" =? "+j2);
    System.out.println(ki+" =? "+k1);    
    System.out.println("hello =? "+s);
    System.out.println("true =? "+b);
    System.out.println("a =? "+a);
  }
}

/*
 * $Log: Marshalling.java,v $
 * Revision 1.6  1997/11/21 21:38:01  mwh
 * Fixed marshalling to respect network byte order.  Added additional
 * classes to support ANON.
 *
 * Revision 1.5  1997/10/21 11:55:30  pankaj
 * Added putShort and getShort.
 *
 * Revision 1.4  1997/10/21 06:00:48  pankaj
 * Fixed errors.
 *
 * Revision 1.2  1997/10/17 17:36:21  pankaj
 * *** empty log message ***
 *
 */
