package ANEP;

/*
 * $Id: SchemeAddrIPv4.java,v 1.1 1997/11/21 21:38:01 mwh Exp $
 * $Revision: 1.1 $
 *
 * Copyright 1997 by Carl A. Gunter, Mike Hicks, Pankaj Kakkar,
 * Jonathan Moore, Scott Nettles
 *
 */

import java.net.*;
import java.io.*;

public class SchemeAddrIPv4 extends SchemeAddr {

  protected InetAddress addr;

  public SchemeAddrIPv4 (InetAddress addr) {
    this.addr = addr;
    schemeType = IPv4;
  }

  public byte[] marshall() {
    byte[] value = new byte [8];
    marshall(value,0);
    return value;
  }

  public void marshall(byte[] value, int offset) {
    if (value.length - offset < 8)
      return; /* XXX make exception */

    // set the address scheme to IPv4 (XXX assumes IPv4 const is <256)
    value[offset] = value[offset+1] = value[offset+2] = 0;
    value[offset+3] = IPv4;

    // write the address
    byte [] ipAddr = addr.getAddress ();
    value [offset+4] = ipAddr[0];
    value [offset+5] = ipAddr[1];
    value [offset+6] = ipAddr[2];
    value [offset+7] = ipAddr[3];
  }

  static public SchemeAddr unmarshall(byte[] value) 
       throws IOException {
    
    ByteArrayInputStream is = new ByteArrayInputStream(value);

    int schemeType = Marshalling.getInt(is);
    if (schemeType != SchemeAddr.IPv4)
      throw new IOException("SchemeAddrIPv4.unmarshall: invalid scheme type "+
			    schemeType);

    InetAddress addr = Marshalling.getInetAddress(is);
    return new SchemeAddrIPv4(addr);
  }

  public InetAddress getInetAddress() { return addr; }
  public String toString() {
    return "SchemeAddr: "+schemeType+" "+addr;
  }
}
/*
 * $Log: SchemeAddrIPv4.java,v $
 * Revision 1.1  1997/11/21 21:38:01  mwh
 * Initial revision
 *
 *
 */
