package ANEP;

/*
 * $Id: SchemeAddrIPv4Port.java,v 1.1 1997/11/21 21:38:01 mwh Exp $
 * $Revision: 1.1 $
 *
 * Copyright 1997 by Carl A. Gunter, Mike Hicks, Pankaj Kakkar,
 * Jonathan Moore, Scott Nettles
 *
 */

import java.net.*;
import java.io.*;

public class SchemeAddrIPv4Port extends SchemeAddrIPv4 {

  protected int port;

  public SchemeAddrIPv4Port (InetAddress addr, int port, int scheme) {
    super(addr);
    this.schemeType = scheme;
    this.port = port;
  }

  public byte[] marshall() {
    byte[] value = new byte [12];
    marshall(value,0);
    return value;
  }

  public void marshall(byte[] value, int offset) {
    if (value.length - offset < 12)
      return; /* XXX make exception */

    // set the address scheme to IPv4 (XXX assumes schem is <256)
    super.marshall(value,offset);
    value[offset+3] = (byte)schemeType;

    // write the add
    // write the port
    value [8] = (byte)((port >> 8) & 0xff);
    value [9] = (byte)(port & 0xff);
    // value[10] and value[11] are padding
  }

  static public SchemeAddr unmarshall(byte[] value) 
       throws java.io.IOException {
    
    ByteArrayInputStream is = new ByteArrayInputStream(value);

    int schemeType = Marshalling.getInt(is);
    if ((schemeType != SchemeAddr.IPv4UDP) &&
	(schemeType != SchemeAddr.IPv4TCP))
      throw new IOException("SchemeAddrIPv4Port.unmarshall: "+
			    "invalid scheme type "+ schemeType);

    InetAddress addr = Marshalling.getInetAddress(is);
    int port = Marshalling.getAsShort(is);
    return new SchemeAddrIPv4Port(addr,port,schemeType);
  }

  public int getPort() { return port; }
  public String toString() {
    return super.toString() + " " + port;
  }
}
/*
 * $Log: SchemeAddrIPv4Port.java,v $
 * Revision 1.1  1997/11/21 21:38:01  mwh
 * Initial revision
 *
 *
 */
