package Log;
import java.text.*;
import java.io.*;
import java.util.Date;

public class Log {

  public final static int QUIET_LEVEL = 0;
  public final static int ERROR_LEVEL = 1;
  public final static int VERBOSE_LEVEL = 2;
  public final static int DEBUG_LEVEL = 3;

  // set default output to stdout
  private static String logfn = null;
  private static OutputStream logfs = 
    new FileOutputStream(FileDescriptor.out); 
  private static int level = ERROR_LEVEL;
  private static DateFormat df = DateFormat.getDateTimeInstance();
  private static boolean sol = true;

  private static String preamble() {
    Date d = new Date();
    return df.format(d);
  }

  // methods to alter the logging location
  public synchronized static void setLogFile(String logFileName) {
    logfs = null;
    logfn = logFileName;
  }

  // alter the logging level
  public synchronized static void setLevel(int l) {
    level = l;
  }

  // output methods
  //
  // if output is directed to a file, the file is reopened
  // each time; this allows the file to be removed/truncated
  // at any time without disturbing the logging function

  public synchronized static void print(int priority, String s) {
    //    System.out.println("level = "+level+", priority = "+priority);
    if (priority <= level) {
      try {
        if (logfn != null) {
          logfs = new FileOutputStream(logfn,true);
	  if (sol)
            logfs.write((preamble()+": ").getBytes());
	  logfs.write(s.getBytes());
          logfs.close();
	  sol = false;
          logfs = null;
        }
        else {
	  if (sol)
	    logfs.write((preamble()+": ").getBytes());
	  logfs.write(s.getBytes());
	  sol = false;
        }
      } catch (IOException e) {
      }
    }
  }
		
  public synchronized static void println(int priority, String s) {
    if (priority <= level) {
      try {
        if (logfn != null) {
          logfs = new FileOutputStream(logfn,true);
	  if (sol)
            logfs.write((preamble()+": ").getBytes());
	  logfs.write((s+"\n").getBytes());
          logfs.flush();
          logfs.close();
	  sol = true;
          logfs = null;
        }
        else {
	  if (sol)
	    logfs.write((preamble()+": ").getBytes());
	  logfs.write((s+"\n").getBytes());
          logfs.flush();
	  sol = true;
        }
      } catch (IOException e) {
      }
    }
  }

  public static void print(String s) {
    print(VERBOSE_LEVEL,s);
  }
		
  public static void println(String s) {
    println(VERBOSE_LEVEL,s);
  }
}




